
# Section: Loading packages -----

rm(list = ls())
#dev.off()
cat("\014")

library("fBasics")
library("gogarch")
library("Jmisc")
library("Matrix")
library("pracma")


# Section: loading helpers -----

function_LastTickInterpolation<-function(ftradetime,fobs){
  
  ffirsthour <- floor(ftradetime[1]/100)
  flasthour  <- floor(ftradetime[NROW(ftradetime)]/100)
  
  ffirstMinute  <- ftradetime[1] - ffirsthour*100
  flastMinute   <- ftradetime[NROW(ftradetime)] - flasthour*100
  
  fhours  <- flasthour-ffirsthour
  
    if (ffirstMinute==59){
      ftimegrid   <- 59 + ffirsthour*100   
    } else{
      ftimegrid   <- seq(ffirstMinute,59,1) + ffirsthour*100
    }  
  
  fhourseq    <- seq((ffirsthour+1),(ffirsthour+fhours),1)*100
  fminseq     <- seq(0,59,1)
  
    for (fhh in 1:NROW(fhourseq)){
      ftimegrid   <- c(ftimegrid,(fhourseq[fhh]+fminseq)) 
    }  
  
  fprices     <- matrix(0,NROW(ftimegrid),1)
  
    for (ftt in 1:NROW(ftimegrid)){
      if (ftt==1){
        fprice   <- fobs[1] 
      }else{
        fprice   <- subset(fobs, ftradetime <= ftimegrid[ftt])  
      }
      
      fprices[ftt]  <- fprice[NROW(fprice)]   
    }
  
  fobsmat  <- cbind(ftimegrid,fprices)
  fobsmat  <- subset(fobsmat,ftimegrid <= ftradetime[NROW(ftradetime)])  
  
  return(fobsmat)
  
}


# Section: Defining tuning parameters -----

ltrunc  <- 800   # truncating the daily time stamps: lower bound
utrunc  <- 2200  # truncating the daily time stamps: upper bound

urange  <- seq(1/100,1/5,1/100)   # integration range for the SLT
bootsN  <- 999                    # number of bootstrap replications

folder_input  <- "provide data input folder here"
folder_output <- "provide data output folder here"


# Section: Initialization the loop -----

year_start    <- 2005
year_end      <- 2020

  for (yr in year_start:year_end){
  CurYear <- yr      
  
    if (yr==year_start){
      smonth    <- 1
      lmonth    <- 12    
    }else if (yr>year_start & yr <year_end){
      smonth    <- 1
      lmonth    <- 12    
    }else if (yr==year_end){
      smonth    <- 1
      lmonth    <- 12  
    }

  lstring_HFsetfolder   <- paste(c(folder_input,"/",CurYear,"/"),collapse = "") # load data in by year
  lstring_USD_EQ_data   <- paste(c("ES1_INDEX.csv"),collapse = "")              # choosing ES1 data
  lstring_USD_FI_data   <- paste(c("TY1_COMDTY.csv"),collapse = "")             # choosing treasury futures (TY1) data
  
  setwd(lstring_HFsetfolder)
  USD_EQdata      <- read.csv(lstring_USD_EQ_data, header = TRUE, sep = ",",na.strings = "#N/A N/A") 
  USD_FIdata      <- read.csv(lstring_USD_FI_data, header = TRUE, sep = ",",na.strings = "#N/A N/A") 
  
    for (mt in smonth:lmonth){
    CurMonth  <- mt
    
    USD_EQdata_mt   <- subset(USD_EQdata,USD_EQdata[,"month"]==mt)
    USD_FIdata_mt   <- subset(USD_FIdata,USD_FIdata[,"month"]==mt)
    days_unique     <- intersect(USD_EQdata_mt[,"day"],USD_FIdata_mt[,"day"])

      for (dd in 1:NROW(days_unique)){
      dayI    <- days_unique[dd]
  
      # Each of the matrices are sorted as: time ~ price
      USD_EQobs   <- subset(USD_EQdata_mt[,c("time","price")], USD_EQdata_mt[,"day"]==dayI)    
      USD_FIobs   <- subset(USD_FIdata_mt[,c("time","price")], USD_FIdata_mt[,"day"]==dayI)  
      
      firsthour_eq <- floor(USD_EQobs[1,"time"]/100)
      lasthour_eq  <- floor(USD_EQobs[NROW(USD_EQobs),"time"]/100)
      
      firsthour_fi <- floor(USD_FIobs[1,"time"]/100)
      lasthour_fi  <- floor(USD_FIobs[NROW(USD_FIobs),"time"]/100)
      
      if(lasthour_eq > firsthour_eq & lasthour_fi>firsthour_fi){  
      USD_EQobs_int  <- function_LastTickInterpolation(USD_EQobs[,"time"],USD_EQobs[,"price"])
      USD_FIobs_int  <- function_LastTickInterpolation(USD_FIobs[,"time"],USD_FIobs[,"price"])
      
      USD_EQobs_sam  <- subset(USD_EQobs_int,USD_EQobs_int[,1]>ltrunc & USD_EQobs_int[,1]<utrunc)
      USD_FIobs_sam  <- subset(USD_FIobs_int,USD_FIobs_int[,1]>ltrunc & USD_FIobs_int[,1]<utrunc)  
      
        if(NROW(USD_EQobs_sam)==NROW(USD_FIobs_sam)){
          
        sscheme  <- seq(1,NROW(USD_EQobs_sam),2)
          
        USD_EQobs_pri  <- USD_EQobs_sam[sscheme,2] 
        USD_FIobs_pri  <- USD_FIobs_sam[sscheme,2]  
        
        # defining return and cosine observations
        dx  <- diff(log(USD_EQobs_pri))
        dy  <- diff(log(USD_FIobs_pri)) 
        dp  <- dx + dy
        dm  <- dx - dy 
        
        Ns  <- NROW(dx)    
        ks  <- floor(Ns^0.45)*6
        
        if(sum(abs(dx[(Ns-ks+1):Ns]))>0 & sum(abs(dx[(Ns-ks+1):Ns]))>0){
          
        estmat_urange <- matrix(0,NROW(urange),4)
        
          for(uu in 1:NROW(urange)){
          xix_uu   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dx)
          xiy_uu   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dy)
          xip_uu   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dp)
          xim_uu   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dm)  
          
          estx_uu  <- - log(mean(xix_uu[(Ns-ks+1):Ns]))/urange[uu]
          esty_uu  <- - log(mean(xiy_uu[(Ns-ks+1):Ns]))/urange[uu]
          estp_uu  <- - log(mean(xip_uu[(Ns-ks+1):Ns]))/urange[uu]
          estm_uu  <- - log(mean(xim_uu[(Ns-ks+1):Ns]))/urange[uu]
          
          estmat_urange[uu,]  <- c(estx_uu,esty_uu,estp_uu,estm_uu) 
          }
        
        # obtaining the spot estimates
        estvarx  <- mean(estmat_urange[,1])
        estvary  <- mean(estmat_urange[,2])
        estcova  <- mean((estmat_urange[,3] - estmat_urange[,4]))/4
        estcorr  <- estcova/sqrt(estvarx*estvary)
        estbeta  <- estcova/estvarx
        
        # preparing for the bootstrap
        # obtaining bipower variation for jump truncation in the spot variance estimate
        # and subsequently local truncated RV to estimate the spot variance
        bpvmat   <-  matrix(0,ks,4) # block specific bipower variation
        rpvmat   <-  matrix(0,ks,4) # block specific bipower variation
        
          for(jj in 1:ks){
          bpvx_jj   <- sum(abs(dx[(Ns-2*ks+jj):(Ns-ks+jj-1)])*abs(dx[(Ns-2*ks+jj+1):(Ns-ks+jj)]))*(Ns/ks)*(pi/2)  
          bpvy_jj   <- sum(abs(dy[(Ns-2*ks+jj):(Ns-ks+jj-1)])*abs(dy[(Ns-2*ks+jj+1):(Ns-ks+jj)]))*(Ns/ks)*(pi/2)  
          bpvp_jj   <- sum(abs(dp[(Ns-2*ks+jj):(Ns-ks+jj-1)])*abs(dp[(Ns-2*ks+jj+1):(Ns-ks+jj)]))*(Ns/ks)*(pi/2)  
          bpvm_jj   <- sum(abs(dm[(Ns-2*ks+jj):(Ns-ks+jj-1)])*abs(dm[(Ns-2*ks+jj+1):(Ns-ks+jj)]))*(Ns/ks)*(pi/2)  
          
          rpvx_jj   <- sum(subset(abs(dx[(Ns-2*ks+jj+1):(Ns-ks+jj)])^2,abs(dx[(Ns-2*ks+jj+1):(Ns-ks+jj)])<= 7*sqrt(bpvx_jj)*(1/Ns)^0.4))*(Ns/ks)
          rpvy_jj   <- sum(subset(abs(dy[(Ns-2*ks+jj+1):(Ns-ks+jj)])^2,abs(dy[(Ns-2*ks+jj+1):(Ns-ks+jj)])<= 7*sqrt(bpvy_jj)*(1/Ns)^0.4))*(Ns/ks)
          rpvp_jj   <- sum(subset(abs(dp[(Ns-2*ks+jj+1):(Ns-ks+jj)])^2,abs(dp[(Ns-2*ks+jj+1):(Ns-ks+jj)])<= 7*sqrt(bpvp_jj)*(1/Ns)^0.4))*(Ns/ks)
          rpvm_jj   <- sum(subset(abs(dm[(Ns-2*ks+jj+1):(Ns-ks+jj)])^2,abs(dm[(Ns-2*ks+jj+1):(Ns-ks+jj)])<= 7*sqrt(bpvm_jj)*(1/Ns)^0.4))*(Ns/ks)
          
          bpvmat[jj,] <- c(bpvx_jj,bpvy_jj,bpvp_jj,bpvm_jj) 
          rpvmat[jj,] <- c(rpvx_jj,rpvy_jj,rpvp_jj,rpvm_jj) 
          }
        
        set.seed(1)  # fixing the bootstrap seed.
        
          for(jj in 1:NROW(rpvmat)){
            for (cc in 1:NCOL(rpvmat)){
              if(rpvmat[jj,cc]==0){
              rpvmat[jj,cc]  <- mean(subset(rpvmat[,cc],rpvmat[,cc]>0))  
              }
            }
          }
        
        etax   <- matrix(rnorm(ks*bootsN),ks,bootsN)
        etay   <- matrix(rnorm(ks*bootsN),ks,bootsN)
        
        rpvx   <- rpvmat[,1]
        rpvy   <- rpvmat[,2]
        rpvc   <- (rpvmat[,3]-rpvmat[,4])/4   # computing the covariance; needed for the resample.
        
        estmat_boot  <- matrix(0,bootsN,4)
        expmat_boot  <- matrix(0,bootsN,4)
        
          for (bb in 1:bootsN){
          dxb   <- sqrt(1/Ns)*sqrt(rpvx)*etax[,bb] 
          dyb1  <- rpvc/sqrt(rpvx)*etax[,bb]
          dyb2  <- sqrt(abs(rpvy-rpvc^2/rpvx))*etay[,bb]  
          dyb   <- sqrt(1/Ns)*(dyb1 + dyb2)
          dpb   <- dxb + dyb
          dmb   <- dxb - dyb
          
          estmat_urangeb  <- matrix(0,NROW(urange),3)   # capturing the bootstrap estimates
          expmat_urangeb  <- matrix(0,NROW(urange),3)   # capturing the bootstrap expectations
            
            for(ub in 1:NROW(urange)){
            xix_uub   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dxb)
            xiy_uub   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dyb)
            xip_uub   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dpb)
            xim_uub   <- cos(sqrt(2*urange[uu])*sqrt(Ns)*dmb)  
            
            estx_uub  <- - log(mean(xix_uub))/urange[uu]
            esty_uub  <- - log(mean(xiy_uub))/urange[uu]
            estp_uub  <- - log(mean(xip_uub))/urange[uu]
            estm_uub  <- - log(mean(xim_uub))/urange[uu]
            
            expx_uub  <- - log(mean(exp(-urange[uu]*rpvx)))/urange[uu]
            expy_uub  <- - log(mean(exp(-urange[uu]*rpvy)))/urange[uu]
            expp_uub  <- - log(mean(exp(-urange[uu]*(rpvx+rpvy+2*rpvc))))/urange[uu]
            expm_uub  <- - log(mean(exp(-urange[uu]*(rpvx+rpvy-2*rpvc))))/urange[uu]
            
            estmat_urangeb[ub,]  <- c(estx_uub,esty_uub,(estp_uub-estm_uub)/4) 
            expmat_urangeb[ub,]  <- c(expx_uub,expy_uub,(expp_uub-expm_uub)/4) 
            } # integration loop
          
          estvarx_bb  <- mean(estmat_urangeb[,1])
          estvary_bb  <- mean(estmat_urangeb[,2])
          estcorr_bb  <- mean(estmat_urangeb[,3])/sqrt(estvarx_bb*estvary_bb)
          estbeta_bb  <- mean(estmat_urangeb[,3])/estvarx_bb
          
          expvarx_bb  <- mean(expmat_urangeb[,1])
          expvary_bb  <- mean(expmat_urangeb[,2])
          expcorr_bb  <- mean(expmat_urangeb[,3])/sqrt(expvarx_bb*expvary_bb)
          expbeta_bb  <- mean(expmat_urangeb[,3])/expvarx_bb
          
          estmat_boot[bb,]  <- c(estvarx_bb,estvary_bb,estcorr_bb,estbeta_bb)
          expmat_boot[bb,]  <- c(expvarx_bb,expvary_bb,expcorr_bb,expbeta_bb)
          
          } # bootstrap loop
        
        boot_quan_vol1  <- sqrt(ks)*(sqrt(estmat_boot[,1])-sqrt(expmat_boot[,1]))
        boot_quan_vol2  <- sqrt(ks)*(sqrt(estmat_boot[,2])-sqrt(expmat_boot[,2]))
        boot_quan_corr  <- sqrt(ks)*(estmat_boot[,3]-expmat_boot[,3])
        boot_quan_beta  <- sqrt(ks)*(estmat_boot[,4]-expmat_boot[,4])
        
        quan_vol1  <- quantile(abs(boot_quan_vol1),0.95)*sqrt(1/ks)
        quan_vol2  <- quantile(abs(boot_quan_vol2),0.95)*sqrt(1/ks)
        quan_corr  <- quantile(abs(boot_quan_corr),0.95)*sqrt(1/ks)
        quan_beta  <- quantile(abs(boot_quan_beta),0.95)*sqrt(1/ks)
        
        stdv_vol1  <- stdev(boot_quan_vol1)
        stdv_vol2  <- stdev(boot_quan_vol2)
        stdv_corr  <- stdev(boot_quan_corr)
        stdv_beta  <- stdev(boot_quan_beta)
        
          if(yr==year_start & mt==smonth & dd==1){
          estimation_vol1  <- t(c(yr,mt,dayI,sqrt(estvarx),sqrt(estvarx)-quan_vol1,sqrt(estvarx)+quan_vol1,stdv_vol1))  
          estimation_vol2  <- t(c(yr,mt,dayI,sqrt(estvary),sqrt(estvary)-quan_vol2,sqrt(estvary)+quan_vol2,stdv_vol2))  
          
          estimation_corr  <- t(c(yr,mt,dayI,estcorr,estcorr-quan_corr,estcorr+quan_corr,stdv_corr)) 
          estimation_beta  <- t(c(yr,mt,dayI,estbeta,estbeta-quan_beta,estbeta+quan_beta,stdv_beta)) 
          }else{
          estimation_vol1  <- rbind(estimation_vol1,t(c(yr,mt,dayI,sqrt(estvarx),sqrt(estvarx)-quan_vol1,sqrt(estvarx)+quan_vol1,stdv_vol1)))  
          estimation_vol2  <- rbind(estimation_vol2,t(c(yr,mt,dayI,sqrt(estvary),sqrt(estvary)-quan_vol2,sqrt(estvary)+quan_vol2,stdv_vol2))) 
          
          estimation_corr  <- rbind(estimation_corr,t(c(yr,mt,dayI,estcorr,estcorr-quan_corr,estcorr+quan_corr,stdv_corr)))
          estimation_beta  <- rbind(estimation_beta,t(c(yr,mt,dayI,estbeta,estbeta-quan_beta,estbeta+quan_beta,stdv_beta)))
          }  
        
        } # end of if condition for return movements
        } # if condition for regular sampling
      } # End of "if" condition for sample truncation
      } # End of day loop 
    } # End of month loop.
  } # End of year loop.

colnames(estimation_vol1)  <- c("year","month","day","vol","lower","upper","stdev")
colnames(estimation_vol2)  <- c("year","month","day","vol","lower","upper","stdev")
colnames(estimation_corr)  <- c("year","month","day","corr","lower","upper","stdev")
colnames(estimation_beta)  <- c("year","month","day","beta","lower","upper","stdev")

setwd(folder_output)

write.csv(estimation_vol1, file = "vol1_data.csv") 
write.csv(estimation_vol2, file = "vol2_data.csv") 
write.csv(estimation_corr, file = "corr_data.csv") 
write.csv(estimation_beta, file = "beta_data.csv") 



