# Section: Loading packages -----

rm(list = ls())
#dev.off()
cat("\014")

library("fBasics")
library("gogarch")
library("Jmisc")
library("Matrix")
library("pracma")
library("OptionPricingRcpp")
library("zoo")
library("MCS")
library("riskParityPortfolio")

  # Packages for Graphics
library("tidyverse")
library("ggplot2")
library("ggcorrplot")
library("gridExtra")
library("xtable")
library("dplyr")

options(scipen = 999)

# Section: Defining tuning parameters -----

folder_input   <- "provide data from output folder from HF data and daily price series here"
folder_output  <- "provide data output folder here for results here"
folder_helpers <- "folder to store helper functions: helper_functions_final.R"


# Section: Source helper functions -----

source(paste0(c(folder_helpers,"/helper_functions_final.R"),collapse = ""  ))


# Section: Loading data -----

setwd(folder_input)

vol1_data  <- read.csv("vol1_data.csv", header = TRUE, sep = ",",na.strings = "NA") 
vol2_data  <- read.csv("vol2_data.csv", header = TRUE, sep = ",",na.strings = "NA") 
corr_data  <- read.csv("corr_data.csv", header = TRUE, sep = ",",na.strings = "NA") 
beta_data  <- read.csv("beta_data.csv", header = TRUE, sep = ",",na.strings = "NA") 
indx_data  <- read.csv("index_data.csv",header = TRUE, sep = ",",na.strings = "NA") 

vol1_data  <- function_data_filter(vol1_data)
vol2_data  <- function_data_filter(vol2_data)
corr_data  <- function_data_filter(corr_data)
beta_data  <- function_data_filter(beta_data)
indx_data  <- function_data_filter(indx_data)

retx_data  <- cbind(indx_data[-1,c("year","month","day")],diff(log(indx_data[,c("ES1.INDEX","TY1.COMDTY")])))

# synchronizing the data
hf_dateX  <- function_dateX_construction(vol1_data[,c("year","month","day")])
dr_dateX  <- function_dateX_construction(retx_data[,c("year","month","day")])
sy_dateX  <- intersect(hf_dateX,dr_dateX)

date_sync  <- function_underlying_sortM(vol1_data[,c("year","month","day")],hf_dateX,sy_dateX)[,1:3]
vol1_sync  <- function_underlying_sortM(vol1_data[,c("vol", "lower","upper","stdev")],hf_dateX,sy_dateX)[,1:4]
vol2_sync  <- function_underlying_sortM(vol2_data[,c("vol", "lower","upper","stdev")],hf_dateX,sy_dateX)[,1:4]
corr_sync  <- function_underlying_sortM(corr_data[,c("corr","lower","upper","stdev")],hf_dateX,sy_dateX)[,1:4]
beta_sync  <- function_underlying_sortM(beta_data[,c("beta","lower","upper","stdev")],hf_dateX,sy_dateX)[,1:4]
retx_sync  <- function_underlying_sortM(retx_data[,c("ES1.INDEX","TY1.COMDTY")],dr_dateX,sy_dateX)[,1:2]

colnames(vol1_sync) <- c("vol", "lower","upper","stdev")
colnames(vol2_sync) <- c("vol", "lower","upper","stdev")
colnames(corr_sync) <- c("corr","lower","upper","stdev")
colnames(beta_sync) <- c("beta","lower","upper","stdev")
colnames(retx_sync) <- c("ES1.INDEX","TY1.COMDTY")
colnames(date_sync) <- c("year","month","day")

  # finding relevant subsamples
indx_data_sub   <- cbind(indx_data[-1,],dr_dateX)
indx_data_2008  <- subset(indx_data_sub,indx_data_sub[,"year"]==2008)
indx_data_2020  <- subset(indx_data_sub,indx_data_sub[,"year"]==2020) 
 


# Section: Unconditional analysis -----

# tail examination
ES1_tail_returns  <- sort(retx_sync[,"ES1.INDEX"])[1:20]

date_sync_estail  <- subset(date_sync, retx_sync[,"ES1.INDEX"] %in% ES1_tail_returns)
retx_sync_estail  <- subset(retx_sync, retx_sync[,"ES1.INDEX"] %in% ES1_tail_returns)

vol1_ES1T_estail  <- subset(vol1_sync,retx_sync[,"ES1.INDEX"] %in% ES1_tail_returns)
vol2_ES1T_estail  <- subset(vol2_sync,retx_sync[,"ES1.INDEX"] %in% ES1_tail_returns)
corr_ES1T_estail  <- subset(corr_sync,retx_sync[,"ES1.INDEX"] %in% ES1_tail_returns)
beta_ES1T_estail  <- subset(beta_sync,retx_sync[,"ES1.INDEX"] %in% ES1_tail_returns)

sumstatmat_sync   <- cbind(retx_sync,vol1_sync[,"vol"],vol2_sync[,"vol"],corr_sync[,"corr"],beta_sync[,"beta"])
sumstatmat_base   <- function_sumstatsbyticker_aug(sumstatmat_sync,1)[,c(1,2,3,4,8)]
  
sumstatmat_base[c(1,2,3,4),c(1,2)]  <-  sumstatmat_base[c(1,2,3,4),c(1,2)]*100     # providing the statistics in pct.

retx_sync_quan    <- cbind(colQuantiles(retx_sync,p=0.025),colQuantiles(retx_sync,p=0.975))*100

boot_sync_quan    <- cbind(colMeans(cbind(vol1_sync[,"lower"],vol2_sync[,"lower"],corr_sync[,"lower"],beta_sync[,"lower"])),
                           colMeans(cbind(vol1_sync[,"upper"],vol2_sync[,"upper"],corr_sync[,"upper"],beta_sync[,"upper"])),
                           colMeans(cbind(vol1_sync[,"stdev"],vol2_sync[,"stdev"],corr_sync[,"stdev"],beta_sync[,"stdev"])))

boot_sync_quan[c(1,2),]    <- boot_sync_quan[c(1,2),]*100    # providing the statistics in pct.

sumstatmat_base_total  <- cbind(sumstatmat_base,rbind(cbind(retx_sync_quan,c(9999,9999)),boot_sync_quan))

rownames(sumstatmat_base_total) <- c("ES1","TY1","vES1","vTY1","corr","beta")  
colnames(sumstatmat_base_total) <- c("mean","stdev","skewness","kurtosis","AR(1)","Q025","Q975","VoV")


# constructing distress-based confidence plots.

  for (ff in 1:NROW(date_sync_estail)){
    if(date_sync_estail[ff,"month"]<10){
      mth_string <- paste(c("0",date_sync_estail[ff,"month"]),collapse = "")  
    }else{
      mth_string <- paste(c(date_sync_estail[ff,"month"]),collapse = "")    
    }
    
    if(date_sync_estail[ff,"day"]<10){
      day_string <- paste(c("0",date_sync_estail[ff,"day"]),collapse = "")  
    }else{
      day_string <- paste(c(date_sync_estail[ff,"day"]),collapse = "")    
    }
    
    if(ff ==1){
      date_sync_estail_string <- paste0(c(date_sync_estail[ff,"year"],"-",mth_string,"-",day_string),collapse = "")   
    }else{
      date_sync_estail_string <- c(date_sync_estail_string,paste0(c(date_sync_estail[ff,"year"],"-",mth_string,"-",day_string),collapse = ""))
    }  
  }



plot_distress_vol1  <-  function_distress_plot2(date_sync_estail_string,vol1_ES1T_estail,sumstatmat_base_total["vES1","mean"]/100,"volatility",c(0,0.120),"ES1")
plot_distress_vol2  <-  function_distress_plot2(date_sync_estail_string,vol2_ES1T_estail,sumstatmat_base_total["vTY1","mean"]/100,"volatility",c(0,0.020),"TY1")

plot_distress_corr  <-  function_distress_plot1(date_sync_estail_string,corr_ES1T_estail,sumstatmat_base_total["corr","mean"],"correlation",c(-1,0.2))
plot_distress_beta  <-  function_distress_plot1(date_sync_estail_string,beta_ES1T_estail,sumstatmat_base_total["beta","mean"],"beta",c(-0.25,0.05))

plot_distress_collect  <- grid.arrange(plot_distress_vol1, plot_distress_vol2, plot_distress_corr, plot_distress_beta, nrow = 2, ncol = 2)
  


# time-varying significance
corr_avg  <- sumstatmat_base_total["corr","mean"]
beta_avg  <- sumstatmat_base_total["beta","mean"]

uniyears  <- unique(date_sync[,"year"])
nomyears  <- NROW(uniyears)

corr_test_mat  <- matrix(0,nomyears,2) 
beta_test_mat  <- matrix(0,nomyears,2) 

  for (yr in 1:nomyears){
  corr_sync_yr  <- subset(corr_sync,date_sync[,"year"]==uniyears[yr])   
  beta_sync_yr  <- subset(beta_sync,date_sync[,"year"]==uniyears[yr]) 
    
  corr_count_dn <- 0
  corr_count_up <- 0
  beta_count_dn <- 0
  beta_count_up <- 0
  
    for(dd in 1:NROW(corr_sync_yr)){
      if (corr_sync_yr[dd,"lower"]>corr_avg){
        corr_count_dn  <- corr_count_dn + 1
      }else if (corr_sync_yr[dd,"upper"]<corr_avg) {
        corr_count_up  <- corr_count_up + 1
      }
      
      if (beta_sync_yr[dd,"lower"]>beta_avg)  {
        beta_count_dn  <- beta_count_dn + 1
      }else if (beta_sync_yr[dd,"upper"]<beta_avg) {
        beta_count_up  <- beta_count_up + 1
      }
    }
  
  corr_test_mat[yr,]  <- t(c(corr_count_dn,corr_count_up))/NROW(corr_sync_yr) 
  beta_test_mat[yr,]  <- t(c(beta_count_dn,beta_count_up))/NROW(beta_sync_yr) 
  
  }

plot_hedge_efficacy_corr <- function_hedge_efficacy(corr_test_mat[,1],corr_test_mat[,2],c("above","below"),"correlation",as.character(uniyears))
plot_hedge_efficacy_beta <- function_hedge_efficacy(beta_test_mat[,1],beta_test_mat[,2],c("above","below"),"beta",as.character(uniyears))


# constructing subsample plots: 2008 financial crisis
indx_data_2008_xv   <- indx_data_2008[,"dr_dateX"]
indx_data_2008_y1   <- indx_data_2008[,"ES1.INDEX"]
indx_data_2008_y2   <- indx_data_2008[,"TY1.COMDTY"]
indx_data_2008_obs  <- data.frame(cbind(indx_data_2008_xv,indx_data_2008_y1,indx_data_2008_y2))

plot_perf_2008 <- ggplot(indx_data_2008_obs,aes(x = indx_data_2008_xv))
plot_perf_2008 <- plot_perf_2008 + geom_line(aes(y = indx_data_2008_y1, colour = "ES1"))
plot_perf_2008 <- plot_perf_2008 + geom_line(aes(y = indx_data_2008_y2*mean(indx_data_2008_y1)/mean(indx_data_2008_y2), colour = "TY1"))
plot_perf_2008 <- plot_perf_2008 + scale_y_continuous(sec.axis = sec_axis(~.*mean(indx_data_2008_y2)/mean(indx_data_2008_y1), name = "TY1 index value"))
plot_perf_2008 <- plot_perf_2008 + scale_colour_manual(values = c("blue", "black"))
plot_perf_2008 <- plot_perf_2008 + labs(title = "performance during 2008",
                                        y = "ES1 index value",
                                        x = "dates",
                                        colour = "Asset: ")
plot_perf_2008 <- plot_perf_2008 + theme(legend.position = "bottom",legend.key.size = unit(1, "cm"))


# constructing subsample plots: 2020 covid crisis
indx_data_2020_xv   <- indx_data_2020[,"dr_dateX"]
indx_data_2020_y1   <- indx_data_2020[,"ES1.INDEX"]
indx_data_2020_y2   <- indx_data_2020[,"TY1.COMDTY"]
indx_data_2020_obs  <- data.frame(cbind(indx_data_2020_xv,indx_data_2020_y1,indx_data_2020_y2))

plot_perf_2020 <- ggplot(indx_data_2020_obs,aes(x = indx_data_2020_xv))
plot_perf_2020 <- plot_perf_2020 + geom_line(aes(y = indx_data_2020_y1, colour = "ES1"))
plot_perf_2020 <- plot_perf_2020 + geom_line(aes(y = indx_data_2020_y2*mean(indx_data_2020_y1)/mean(indx_data_2020_y2), colour = "TY1"))
plot_perf_2020 <- plot_perf_2020 + scale_y_continuous(sec.axis = sec_axis(~.*mean(indx_data_2020_y2)/mean(indx_data_2020_y1), name = "TY1 index value"))
plot_perf_2020 <- plot_perf_2020 + scale_colour_manual(values = c("blue", "black"))
plot_perf_2020 <- plot_perf_2020 + geom_vline(xintercept = 2020.189, colour="orange",size=1,linetype = "dashed")
plot_perf_2020 <- plot_perf_2020 + labs(title = "performance during 2020",
                                        y = "ES1 index value",
                                        x = "dates",
                                        colour = "Asset: ")
plot_perf_2020 <- plot_perf_2020 + theme(legend.position = "bottom",legend.key.size = unit(1, "cm"))



# Second: In-sample HAR Analysis ----

Tpre    <- NROW(vol1_sync)

# defining the variables: ES 1 volatility
vol1_y  <- vol1_sync[22:Tpre,"vol"]
vol1_xd <- rollmean(vol1_sync[,"vol"],  k = 1 )[(22-1) :(Tpre-1)]
vol1_xw <- rollmean(vol1_sync[,"vol"],  k = 5 )[(22-5) :(Tpre-5)]
vol1_xm <- rollmean(vol1_sync[,"vol"],  k = 21)[(22-21):(Tpre-21)]
vol1_rq <- demean(rollmean(vol1_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-1)])

vol1_xar1  <- vol1_xd  
vol1_xarQ  <- cbind(vol1_xd,vol1_xd*vol1_rq)
vol1_xhar  <- cbind(vol1_xd,vol1_xw,vol1_xm)
vol1_xhaQ  <- cbind(vol1_xd,vol1_xw,vol1_xm,vol1_xd*vol1_rq)
vol1_xhQL  <- cbind(vol1_xd,vol1_xw,vol1_xm,vol1_xd*vol1_rq,vol1_xw*vol1_rq,vol1_xm*vol1_rq)

# defining the variables: TY 1 volatility
vol2_y  <- vol2_sync[22:Tpre,"vol"]
vol2_xd <- rollmean(vol2_sync[,"vol"],  k = 1 )[(22-1) :(Tpre-1)]
vol2_xw <- rollmean(vol2_sync[,"vol"],  k = 5 )[(22-5) :(Tpre-5)]
vol2_xm <- rollmean(vol2_sync[,"vol"],  k = 21)[(22-21):(Tpre-21)]
vol2_rq <- demean(rollmean(vol2_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-1)])

vol2_xar1  <- vol2_xd  
vol2_xarQ  <- cbind(vol2_xd,vol2_xd*vol2_rq)
vol2_xhar  <- cbind(vol2_xd,vol2_xw,vol2_xm)
vol2_xhaQ  <- cbind(vol2_xd,vol2_xw,vol2_xm,vol2_xd*vol2_rq)
vol2_xhQL  <- cbind(vol2_xd,vol2_xw,vol2_xm,vol2_xd*vol2_rq,vol2_xw*vol2_rq,vol2_xm*vol2_rq)

# defining the variables: correlation
corr_y  <- corr_sync[22:Tpre,"corr"]
corr_xd <- rollmean(corr_sync[,"corr"], k = 1 )[(22-1) :(Tpre-1)]
corr_xw <- rollmean(corr_sync[,"corr"], k = 5 )[(22-5) :(Tpre-5)]
corr_xm <- rollmean(corr_sync[,"corr"], k = 21)[(22-21):(Tpre-21)]
corr_rq <- demean(rollmean(corr_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-1)])

corr_xar1  <- corr_xd  
corr_xarQ  <- cbind(corr_xd,corr_xd*corr_rq)
corr_xhar  <- cbind(corr_xd,corr_xw,corr_xm)
corr_xhaQ  <- cbind(corr_xd,corr_xw,corr_xm,corr_xd*corr_rq)
corr_xhQL  <- cbind(corr_xd,corr_xw,corr_xm,corr_xd*corr_rq,corr_xw*corr_rq,corr_xm*corr_rq)

# defining the variables: Correlation
beta_y  <- beta_sync[22:Tpre,"beta"]
beta_xd <- rollmean(beta_sync[,"beta"], k = 1 )[(22-1) :(Tpre-1)]
beta_xw <- rollmean(beta_sync[,"beta"], k = 5 )[(22-5) :(Tpre-5)]
beta_xm <- rollmean(beta_sync[,"beta"], k = 21)[(22-21):(Tpre-21)]
beta_rq <- demean(rollmean(beta_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-1)])

beta_xar1  <- beta_xd  
beta_xarQ  <- cbind(beta_xd,beta_xd*beta_rq)
beta_xhar  <- cbind(beta_xd,beta_xw,beta_xm)
beta_xhaQ  <- cbind(beta_xd,beta_xw,beta_xm,beta_xd*beta_rq)
beta_xhQL  <- cbind(beta_xd,beta_xw,beta_xm,beta_xd*beta_rq,beta_xw*beta_rq,beta_xm*beta_rq)

# regressions for the various models
vol1_regs_ar1  <- function_predictive_regression_evaluation(vol1_y,vol1_xar1)
vol1_regs_arQ  <- function_predictive_regression_evaluation(vol1_y,vol1_xarQ)
vol1_regs_har  <- function_predictive_regression_evaluation(vol1_y,vol1_xhar)
vol1_regs_haQ  <- function_predictive_regression_evaluation(vol1_y,vol1_xhaQ)
vol1_regs_hQL  <- function_predictive_regression_evaluation(vol1_y,vol1_xhQL)

vol2_regs_ar1  <- function_predictive_regression_evaluation(vol2_y,vol2_xar1)
vol2_regs_arQ  <- function_predictive_regression_evaluation(vol2_y,vol2_xarQ)
vol2_regs_har  <- function_predictive_regression_evaluation(vol2_y,vol2_xhar)
vol2_regs_haQ  <- function_predictive_regression_evaluation(vol2_y,vol2_xhaQ)
vol2_regs_hQL  <- function_predictive_regression_evaluation(vol2_y,vol2_xhQL)

corr_regs_ar1  <- function_predictive_regression_evaluation(corr_y,corr_xar1)
corr_regs_arQ  <- function_predictive_regression_evaluation(corr_y,corr_xarQ)
corr_regs_har  <- function_predictive_regression_evaluation(corr_y,corr_xhar)
corr_regs_haQ  <- function_predictive_regression_evaluation(corr_y,corr_xhaQ)
corr_regs_hQL  <- function_predictive_regression_evaluation(corr_y,corr_xhQL)

beta_regs_ar1  <- function_predictive_regression_evaluation(beta_y,beta_xar1)
beta_regs_arQ  <- function_predictive_regression_evaluation(beta_y,beta_xarQ)
beta_regs_har  <- function_predictive_regression_evaluation(beta_y,beta_xhar)
beta_regs_haQ  <- function_predictive_regression_evaluation(beta_y,beta_xhaQ)
beta_regs_hQL  <- function_predictive_regression_evaluation(beta_y,beta_xhQL)

# collecting results that will later be split
results_regs_ar1  <- cbind(vol1_regs_ar1,vol2_regs_ar1,corr_regs_ar1,beta_regs_ar1)[1:5,]
results_regs_arQ  <- cbind(vol1_regs_arQ,vol2_regs_arQ,corr_regs_arQ,beta_regs_arQ)[1:7,]
results_regs_har  <- cbind(vol1_regs_har,vol2_regs_har,corr_regs_har,beta_regs_har)[1:9,]
results_regs_haQ  <- cbind(vol1_regs_haQ,vol2_regs_haQ,corr_regs_haQ,beta_regs_haQ)[1:11,]
results_regs_hQL  <- cbind(vol1_regs_hQL,vol2_regs_hQL,corr_regs_hQL,beta_regs_hQL)[1:15,]

colnames(results_regs_ar1) <- c("ES1","TY1","corr","beta")
colnames(results_regs_arQ) <- c("ES1","TY1","corr","beta")
colnames(results_regs_har) <- c("ES1","TY1","corr","beta")
colnames(results_regs_haQ) <- c("ES1","TY1","corr","beta")
colnames(results_regs_hQL) <- c("ES1","TY1","corr","beta")

rownames(results_regs_ar1)  <- c("c","HARd","t-c","t-HARd","R2")
rownames(results_regs_arQ)  <- c("c","HARd","HARdQ","t-c","t-HARd","t-HARdQ","R2")
rownames(results_regs_har)  <- c("c","HARd","HARw","HARm","t-c","t-HARd","t-HARw","t-HARm","R2")
rownames(results_regs_haQ)  <- c("c","HARd","HARw","HARm","HARdQ","t-c","t-HARd","t-HARw","t-HARm","t-HARdQ","R2")
rownames(results_regs_hQL)  <- c("c","HARd","HARw","HARm","HARdQ","HARwQ","HARmQ","t-c","t-HARd","t-HARw","t-HARm","t-HARdQ","t-HARwQ","t-HARmQ","R2")



# Section: OOS HAR analysis -----

fsteps  <- 21
spread  <- 10/10000   # spread in basis points

# defining the variables again with the demeaning
# defining the variables: ES 1 volatility
vol1_y  <- rollmean(vol1_sync[,"vol"],  k = fsteps)[22: (Tpre-fsteps+1)]
vol1_xd <- rollmean(vol1_sync[,"vol"],  k = 1 )[(22-1) :(Tpre-fsteps+1-1)]
vol1_xw <- rollmean(vol1_sync[,"vol"],  k = 5 )[(22-5) :(Tpre-fsteps+1-5)]
vol1_xm <- rollmean(vol1_sync[,"vol"],  k = 21)[(22-21):(Tpre-fsteps+1-21)]
vol1_rq <- rollmean(vol1_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-fsteps+1-1)]

vol1_xar1  <- vol1_xd  
vol1_xarQ  <- cbind(vol1_xd,vol1_xd*vol1_rq)
vol1_xhar  <- cbind(vol1_xd,vol1_xw,vol1_xm)
vol1_xhaQ  <- cbind(vol1_xd,vol1_xw,vol1_xm,vol1_xd*vol1_rq)
vol1_xhQL  <- cbind(vol1_xd,vol1_xw,vol1_xm,vol1_xd*vol1_rq,vol1_xw*vol1_rq,vol1_xm*vol1_rq)

# defining the variables: TY 1 volatility
vol2_y  <- rollmean(vol2_sync[,"vol"],  k = fsteps)[22: (Tpre-fsteps+1)]
vol2_xd <- rollmean(vol2_sync[,"vol"],  k = 1 )[(22-1) :(Tpre-fsteps+1-1)]
vol2_xw <- rollmean(vol2_sync[,"vol"],  k = 5 )[(22-5) :(Tpre-fsteps+1-5)]
vol2_xm <- rollmean(vol2_sync[,"vol"],  k = 21)[(22-21):(Tpre-fsteps+1-21)]
vol2_rq <- rollmean(vol2_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-fsteps+1-1)]

vol2_xar1  <- vol2_xd  
vol2_xarQ  <- cbind(vol2_xd,vol2_xd*vol2_rq)
vol2_xhar  <- cbind(vol2_xd,vol2_xw,vol2_xm)
vol2_xhaQ  <- cbind(vol2_xd,vol2_xw,vol2_xm,vol2_xd*vol2_rq)
vol2_xhQL  <- cbind(vol2_xd,vol2_xw,vol2_xm,vol2_xd*vol2_rq,vol2_xw*vol2_rq,vol2_xm*vol2_rq)

# defining the variables: correlation
corr_y  <- rollmean(corr_sync[,"corr"], k = fsteps)[22: (Tpre-fsteps+1)]
corr_xd <- rollmean(corr_sync[,"corr"], k = 1 )[(22-1) :(Tpre-fsteps+1-1)]
corr_xw <- rollmean(corr_sync[,"corr"], k = 5 )[(22-5) :(Tpre-fsteps+1-5)]
corr_xm <- rollmean(corr_sync[,"corr"], k = 21)[(22-21):(Tpre-fsteps+1-21)]
corr_rq <- rollmean(corr_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-fsteps+1-1)]

corr_xar1  <- corr_xd  
corr_xarQ  <- cbind(corr_xd,corr_xd*corr_rq)
corr_xhar  <- cbind(corr_xd,corr_xw,corr_xm)
corr_xhaQ  <- cbind(corr_xd,corr_xw,corr_xm,corr_xd*corr_rq)
corr_xhQL  <- cbind(corr_xd,corr_xw,corr_xm,corr_xd*corr_rq,corr_xw*corr_rq,corr_xm*corr_rq)

# defining the variables: Correlation
beta_y  <- rollmean(beta_sync[,"beta"], k = fsteps)[22: (Tpre-fsteps+1)]
beta_xd <- rollmean(beta_sync[,"beta"], k = 1 )[(22-1) :(Tpre-fsteps+1-1)]
beta_xw <- rollmean(beta_sync[,"beta"], k = 5 )[(22-5) :(Tpre-fsteps+1-5)]
beta_xm <- rollmean(beta_sync[,"beta"], k = 21)[(22-21):(Tpre-fsteps+1-21)]
beta_rq <- rollmean(beta_sync[,"stdev"],k = 1 )[(22-1) :(Tpre-fsteps+1-1)]

beta_xar1  <- beta_xd  
beta_xarQ  <- cbind(beta_xd,beta_xd*beta_rq)
beta_xhar  <- cbind(beta_xd,beta_xw,beta_xm)
beta_xhaQ  <- cbind(beta_xd,beta_xw,beta_xm,beta_xd*beta_rq)
beta_xhQL  <- cbind(beta_xd,beta_xw,beta_xm,beta_xd*beta_rq,beta_xw*beta_rq,beta_xm*beta_rq)

# defining return variables
retx_y  <- rollmean(retx_sync, k = fsteps)[22: (Tpre-fsteps+1),]


# designing the OOS exercise
Tsam    <- NROW(vol1_y)
Tini    <- 250
Tout    <- Tsam - Tini
scheme  <- "expanding"
Tmix    <- 1000          # if choosing a mixture between rolling and expanding; cuts when going from expanding to rolling

vol1_eror_mat <- matrix(0,Tout,5)   # five models being tested
vol2_eror_mat <- matrix(0,Tout,5)   # five models being tested
corr_eror_mat <- matrix(0,Tout,5)   # five models being tested
beta_eror_mat <- matrix(0,Tout,5)   # five models being tested
beta_pred_mat <- matrix(0,Tout,5)   # five models being tested
retx_perf_mat <- matrix(0,Tout,5)   # five models being tested
beta_revs_mat <- matrix(0,Tout,5)   # five models being tested
retx_revs_mat <- matrix(0,Tout,5)   # five models being tested
wght_rpar_mat <- matrix(0,Tout,10)  # since there are two weights 
retx_rpar_mat <- matrix(0,Tout,5)   # five models being tested


  for(tt in 1:Tout){
    if(scheme == "expanding"){
      spar <- 1  
    }else if (scheme == "rolling"){
      spar <- tt  
    }else if (scheme == "mixed"){
      if(tt<=Tmix){
        spar <- 1  
      }else{
        spar <- tt-Tmix   
      }
    }
    
  # ES1 forecasting  
  vol1_y_tt_in      <- vol1_y[spar:(Tini+tt-1)]
  vol1_xar1_tt_in   <- vol1_xar1[spar:(Tini+tt-1)]
  vol1_xarQ_tt_in   <- vol1_xarQ[spar:(Tini+tt-1),]
  vol1_xhar_tt_in   <- vol1_xhar[spar:(Tini+tt-1),]
  vol1_xhaQ_tt_in   <- vol1_xhaQ[spar:(Tini+tt-1),]
  vol1_xhQL_tt_in   <- vol1_xhQL[spar:(Tini+tt-1),]  
  
  vol1_y_tt_out     <- vol1_y[(Tini+tt)]
  vol1_xar1_tt_out  <- vol1_xar1[(Tini+tt)]
  vol1_xarQ_tt_out  <- vol1_xarQ[(Tini+tt),]
  vol1_xhar_tt_out  <- vol1_xhar[(Tini+tt),]
  vol1_xhaQ_tt_out  <- vol1_xhaQ[(Tini+tt),]
  vol1_xhQL_tt_out  <- vol1_xhQL[(Tini+tt),]  
  
  vol1_regs_ar1_tt  <- function_predictive_regression_evaluation(vol1_y_tt_in,vol1_xar1_tt_in)[1:2]
  vol1_regs_arQ_tt  <- function_predictive_regression_evaluation(vol1_y_tt_in,vol1_xarQ_tt_in)[1:3]
  vol1_regs_har_tt  <- function_predictive_regression_evaluation(vol1_y_tt_in,vol1_xhar_tt_in)[1:4]
  vol1_regs_haQ_tt  <- function_predictive_regression_evaluation(vol1_y_tt_in,vol1_xhaQ_tt_in)[1:5]
  vol1_regs_hQL_tt  <- function_predictive_regression_evaluation(vol1_y_tt_in,vol1_xhQL_tt_in)[1:7]
  
  vol1_pred_ar1_tt  <- t(vol1_regs_ar1_tt) %*% c(1,vol1_xar1_tt_out)
  vol1_pred_arQ_tt  <- t(vol1_regs_arQ_tt) %*% c(1,vol1_xarQ_tt_out)
  vol1_pred_har_tt  <- t(vol1_regs_har_tt) %*% c(1,vol1_xhar_tt_out)
  vol1_pred_haQ_tt  <- t(vol1_regs_haQ_tt) %*% c(1,vol1_xhaQ_tt_out)
  vol1_pred_hQL_tt  <- t(vol1_regs_hQL_tt) %*% c(1,vol1_xhQL_tt_out)
  
  vol1_eror_mat[tt,]  <- t(rep(vol1_y_tt_out,5) - c(vol1_pred_ar1_tt,vol1_pred_arQ_tt,vol1_pred_har_tt,vol1_pred_haQ_tt,vol1_pred_hQL_tt))
  
  # TY1 forecasting
  vol2_y_tt_in      <- vol2_y[spar:(Tini+tt-1)]
  vol2_xar1_tt_in   <- vol2_xar1[spar:(Tini+tt-1)]
  vol2_xarQ_tt_in   <- vol2_xarQ[spar:(Tini+tt-1),]
  vol2_xhar_tt_in   <- vol2_xhar[spar:(Tini+tt-1),]
  vol2_xhaQ_tt_in   <- vol2_xhaQ[spar:(Tini+tt-1),]
  vol2_xhQL_tt_in   <- vol2_xhQL[spar:(Tini+tt-1),]  
  
  vol2_y_tt_out     <- vol2_y[(Tini+tt)]
  vol2_xar1_tt_out  <- vol2_xar1[(Tini+tt)]
  vol2_xarQ_tt_out  <- vol2_xarQ[(Tini+tt),]
  vol2_xhar_tt_out  <- vol2_xhar[(Tini+tt),]
  vol2_xhaQ_tt_out  <- vol2_xhaQ[(Tini+tt),]
  vol2_xhQL_tt_out  <- vol2_xhQL[(Tini+tt),]  
  
  vol2_regs_ar1_tt  <- function_predictive_regression_evaluation(vol2_y_tt_in,vol2_xar1_tt_in)[1:2]
  vol2_regs_arQ_tt  <- function_predictive_regression_evaluation(vol2_y_tt_in,vol2_xarQ_tt_in)[1:3]
  vol2_regs_har_tt  <- function_predictive_regression_evaluation(vol2_y_tt_in,vol2_xhar_tt_in)[1:4]
  vol2_regs_haQ_tt  <- function_predictive_regression_evaluation(vol2_y_tt_in,vol2_xhaQ_tt_in)[1:5]
  vol2_regs_hQL_tt  <- function_predictive_regression_evaluation(vol2_y_tt_in,vol2_xhQL_tt_in)[1:7]
  
  vol2_pred_ar1_tt  <- t(vol2_regs_ar1_tt) %*% c(1,vol2_xar1_tt_out)
  vol2_pred_arQ_tt  <- t(vol2_regs_arQ_tt) %*% c(1,vol2_xarQ_tt_out)
  vol2_pred_har_tt  <- t(vol2_regs_har_tt) %*% c(1,vol2_xhar_tt_out)
  vol2_pred_haQ_tt  <- t(vol2_regs_haQ_tt) %*% c(1,vol2_xhaQ_tt_out)
  vol2_pred_hQL_tt  <- t(vol2_regs_hQL_tt) %*% c(1,vol2_xhQL_tt_out)
  
  vol2_eror_mat[tt,]  <- t(rep(vol2_y_tt_out,5) - c(vol2_pred_ar1_tt,vol2_pred_arQ_tt,vol2_pred_har_tt,vol2_pred_haQ_tt,vol2_pred_hQL_tt))
  
  # Correlation forecasting
  corr_y_tt_in      <- corr_y[spar:(Tini+tt-1)]
  corr_xar1_tt_in   <- corr_xar1[spar:(Tini+tt-1)]
  corr_xarQ_tt_in   <- corr_xarQ[spar:(Tini+tt-1),]
  corr_xhar_tt_in   <- corr_xhar[spar:(Tini+tt-1),]
  corr_xhaQ_tt_in   <- corr_xhaQ[spar:(Tini+tt-1),]
  corr_xhQL_tt_in   <- corr_xhQL[spar:(Tini+tt-1),]  
  
  corr_y_tt_out     <- corr_y[(Tini+tt)]
  corr_xar1_tt_out  <- corr_xar1[(Tini+tt)]
  corr_xarQ_tt_out  <- corr_xarQ[(Tini+tt),]
  corr_xhar_tt_out  <- corr_xhar[(Tini+tt),]
  corr_xhaQ_tt_out  <- corr_xhaQ[(Tini+tt),]
  corr_xhQL_tt_out  <- corr_xhQL[(Tini+tt),]  
  
  corr_regs_ar1_tt  <- function_predictive_regression_evaluation(corr_y_tt_in,corr_xar1_tt_in)[1:2]
  corr_regs_arQ_tt  <- function_predictive_regression_evaluation(corr_y_tt_in,corr_xarQ_tt_in)[1:3]
  corr_regs_har_tt  <- function_predictive_regression_evaluation(corr_y_tt_in,corr_xhar_tt_in)[1:4]
  corr_regs_haQ_tt  <- function_predictive_regression_evaluation(corr_y_tt_in,corr_xhaQ_tt_in)[1:5]
  corr_regs_hQL_tt  <- function_predictive_regression_evaluation(corr_y_tt_in,corr_xhQL_tt_in)[1:7]
  
  corr_pred_ar1_tt  <- t(corr_regs_ar1_tt) %*% c(1,corr_xar1_tt_out)
  corr_pred_arQ_tt  <- t(corr_regs_arQ_tt) %*% c(1,corr_xarQ_tt_out)
  corr_pred_har_tt  <- t(corr_regs_har_tt) %*% c(1,corr_xhar_tt_out)
  corr_pred_haQ_tt  <- t(corr_regs_haQ_tt) %*% c(1,corr_xhaQ_tt_out)
  corr_pred_hQL_tt  <- t(corr_regs_hQL_tt) %*% c(1,corr_xhQL_tt_out)
  
  corr_eror_mat[tt,]  <- t(rep(corr_y_tt_out,5) - c(corr_pred_ar1_tt,corr_pred_arQ_tt,corr_pred_har_tt,corr_pred_haQ_tt,corr_pred_hQL_tt))
  
  # beta forecasting
  beta_y_tt_in      <- beta_y[spar:(Tini+tt-1)]
  beta_xar1_tt_in   <- beta_xar1[spar:(Tini+tt-1)]
  beta_xarQ_tt_in   <- beta_xarQ[spar:(Tini+tt-1),]
  beta_xhar_tt_in   <- beta_xhar[spar:(Tini+tt-1),]
  beta_xhaQ_tt_in   <- beta_xhaQ[spar:(Tini+tt-1),]
  beta_xhQL_tt_in   <- beta_xhQL[spar:(Tini+tt-1),]  
  
  beta_y_tt_out     <- beta_y[(Tini+tt)]
  beta_xar1_tt_out  <- beta_xar1[(Tini+tt)]
  beta_xarQ_tt_out  <- beta_xarQ[(Tini+tt),]
  beta_xhar_tt_out  <- beta_xhar[(Tini+tt),]
  beta_xhaQ_tt_out  <- beta_xhaQ[(Tini+tt),]
  beta_xhQL_tt_out  <- beta_xhQL[(Tini+tt),]  
  
  beta_regs_ar1_tt  <- function_predictive_regression_evaluation(beta_y_tt_in,beta_xar1_tt_in)[1:2]
  beta_regs_arQ_tt  <- function_predictive_regression_evaluation(beta_y_tt_in,beta_xarQ_tt_in)[1:3]
  beta_regs_har_tt  <- function_predictive_regression_evaluation(beta_y_tt_in,beta_xhar_tt_in)[1:4]
  beta_regs_haQ_tt  <- function_predictive_regression_evaluation(beta_y_tt_in,beta_xhaQ_tt_in)[1:5]
  beta_regs_hQL_tt  <- function_predictive_regression_evaluation(beta_y_tt_in,beta_xhQL_tt_in)[1:7]
  
  beta_pred_ar1_tt  <- t(beta_regs_ar1_tt) %*% c(1,beta_xar1_tt_out)
  beta_pred_arQ_tt  <- t(beta_regs_arQ_tt) %*% c(1,beta_xarQ_tt_out)
  beta_pred_har_tt  <- t(beta_regs_har_tt) %*% c(1,beta_xhar_tt_out)
  beta_pred_haQ_tt  <- t(beta_regs_haQ_tt) %*% c(1,beta_xhaQ_tt_out)
  beta_pred_hQL_tt  <- t(beta_regs_hQL_tt) %*% c(1,beta_xhQL_tt_out)
  
  beta_pred_mat[tt,]  <- t(c(beta_pred_ar1_tt,beta_pred_arQ_tt,beta_pred_har_tt,beta_pred_haQ_tt,beta_pred_hQL_tt))
  beta_eror_mat[tt,]  <- t(rep(beta_y_tt_out,5) - c(beta_pred_ar1_tt,beta_pred_arQ_tt,beta_pred_har_tt,beta_pred_haQ_tt,beta_pred_hQL_tt))
  
  beta_revs_mat[tt,]  <- t(c(corr_pred_ar1_tt*vol1_pred_ar1_tt/vol2_pred_ar1_tt,
                             corr_pred_arQ_tt*vol1_pred_arQ_tt/vol2_pred_arQ_tt,
                             corr_pred_har_tt*vol1_pred_har_tt/vol2_pred_har_tt,
                             corr_pred_haQ_tt*vol1_pred_haQ_tt/vol2_pred_haQ_tt,
                             corr_pred_hQL_tt*vol1_pred_hQL_tt/vol2_pred_hQL_tt))
  
  wght_rpar_mat[tt,]  <- t(c(riskParityPortfolio(unvech(c(vol1_pred_ar1_tt^2,corr_pred_ar1_tt*vol1_pred_ar1_tt*vol2_pred_ar1_tt,vol2_pred_ar1_tt^2)))$w,
                             riskParityPortfolio(unvech(c(vol1_pred_arQ_tt^2,corr_pred_arQ_tt*vol1_pred_arQ_tt*vol2_pred_arQ_tt,vol2_pred_arQ_tt^2)))$w,
                             riskParityPortfolio(unvech(c(vol1_pred_har_tt^2,corr_pred_har_tt*vol1_pred_har_tt*vol2_pred_har_tt,vol2_pred_har_tt^2)))$w,
                             riskParityPortfolio(unvech(c(vol1_pred_haQ_tt^2,corr_pred_haQ_tt*vol1_pred_haQ_tt*vol2_pred_haQ_tt,vol2_pred_haQ_tt^2)))$w,
                             riskParityPortfolio(unvech(c(vol1_pred_hQL_tt^2,corr_pred_hQL_tt*vol1_pred_hQL_tt*vol2_pred_hQL_tt,vol2_pred_hQL_tt^2)))$w))

  retx_y_tt_out  <- retx_y[(Tini+tt),]
  
    if(tt==1)
    {
    retx_perf_mat[tt,]  <- t(rep(retx_y_tt_out[2],5) - retx_y_tt_out[1]*beta_pred_mat[tt,])
    retx_revs_mat[tt,]  <- t(rep(retx_y_tt_out[1],5) - retx_y_tt_out[2]*beta_revs_mat[tt,])
    retx_rpar_mat[tt,]  <- t(wght_rpar_mat[tt,c(1,3,5,7,9)]*retx_y_tt_out[1] + wght_rpar_mat[tt,c(2,4,6,8,10)]*retx_y_tt_out[2])
    }else
    {
    retx_perf_mat[tt,]  <- t(rep(retx_y_tt_out[2],5) - retx_y_tt_out[1]*beta_pred_mat[tt,]) - abs(beta_pred_mat[tt,]-beta_pred_mat[(tt-1),])*spread 
    retx_revs_mat[tt,]  <- t(rep(retx_y_tt_out[1],5) - retx_y_tt_out[2]*beta_revs_mat[tt,]) - abs(beta_revs_mat[tt,]-beta_revs_mat[(tt-1),])*spread 
    retx_rpar_mat[tt,]  <- t(wght_rpar_mat[tt,c(1,3,5,7,9)]*retx_y_tt_out[1] + wght_rpar_mat[tt,c(2,4,6,8,10)]*retx_y_tt_out[2])
    retx_rpar_mat[tt,]  <- retx_rpar_mat[tt,] - (abs(wght_rpar_mat[tt,c(1,3,5,7,9)]-wght_rpar_mat[(tt-1),c(1,3,5,7,9)]) + abs(wght_rpar_mat[tt,c(2,4,6,8,10)]-wght_rpar_mat[(tt-1),c(2,4,6,8,10)]))*spread
    }
  
  }

forecast_eMat_vol1  <- vol1_eror_mat^2
forecast_eMat_vol2  <- vol2_eror_mat^2
forecast_eMat_corr  <- corr_eror_mat^2
forecast_eMat_beta  <- beta_eror_mat^2
rmse_eMat_collect   <- t(cbind((sqrt(colMeans(forecast_eMat_vol1))*100),
                               (sqrt(colMeans(forecast_eMat_vol2))*100),
                               (sqrt(colMeans(forecast_eMat_corr))*100),
                               (sqrt(colMeans(forecast_eMat_beta))*100)))

colnames(rmse_eMat_collect)  <- c("AR","ARQ","HAR","HARQ","HARQF")
rownames(rmse_eMat_collect)  <- c("vES1","vTY1","corr","beta")

colnames(forecast_eMat_vol1) <- c("AR","ARQ","HAR","HARQ","HARQF")
colnames(forecast_eMat_vol2) <- c("AR","ARQ","HAR","HARQ","HARQF")
colnames(forecast_eMat_corr) <- c("AR","ARQ","HAR","HARQ","HARQF")
colnames(forecast_eMat_beta) <- c("AR","ARQ","HAR","HARQ","HARQF")

vol1_mcs  <- MCSprocedure(forecast_eMat_vol1,alpha=0.1,B=5000,statistic='Tmax',cl=NULL)
vol2_mcs  <- MCSprocedure(forecast_eMat_vol2,alpha=0.1,B=5000,statistic='Tmax',cl=NULL)
corr_mcs  <- MCSprocedure(forecast_eMat_corr,alpha=0.1,B=5000,statistic='Tmax',cl=NULL)
beta_mcs  <- MCSprocedure(forecast_eMat_beta,alpha=0.1,B=5000,statistic='Tmax',cl=NULL)


# Section: Storing stuff -----

setwd(folder_output)

xTable_sumstatmat_base_total   <- xtable(sumstatmat_base_total, caption = "summary stats",align=c("l","c","c","c","c","c","c","c","c"),digits = 4)
print.xtable(xTable_sumstatmat_base_total, type="latex", file="empirical_sumstats.tex")

ggsave("plot_distress_vol1.png",plot=plot_distress_vol1, width = 14, height = 9.5, units = "cm")
ggsave("plot_distress_vol2.png",plot=plot_distress_vol2, width = 14, height = 9.5, units = "cm")
ggsave("plot_distress_corr.png",plot=plot_distress_corr, width = 14, height = 9.5, units = "cm")
ggsave("plot_distress_beta.png",plot=plot_distress_beta, width = 14, height = 9.5, units = "cm")

ggsave("plot_hedge_efficacy_corr.png",plot=plot_hedge_efficacy_corr, width = 14, height = 9.5, units = "cm")
ggsave("plot_hedge_efficacy_beta.png",plot=plot_hedge_efficacy_beta, width = 14, height = 9.5, units = "cm")

ggsave("plot_perf_2008.png",plot=plot_perf_2008, width = 14, height = 9.5, units = "cm")
ggsave("plot_perf_2020.png",plot=plot_perf_2020, width = 14, height = 9.5, units = "cm")

xTable_results_regs_ar1  <- xtable(results_regs_ar1, caption = "HAR estimates",align=c("l","c","c","c","c"),digits = 4)
xTable_results_regs_arQ  <- xtable(results_regs_arQ, caption = "HAR estimates",align=c("l","c","c","c","c"),digits = 4)
xTable_results_regs_har  <- xtable(results_regs_har, caption = "HAR estimates",align=c("l","c","c","c","c"),digits = 4)
xTable_results_regs_haQ  <- xtable(results_regs_haQ, caption = "HAR estimates",align=c("l","c","c","c","c"),digits = 4)
xTable_results_regs_hQL  <- xtable(results_regs_hQL, caption = "HAR estimates",align=c("l","c","c","c","c"),digits = 4)

print.xtable(xTable_results_regs_ar1, type="latex", file="har_analysis_ar1.tex")
print.xtable(xTable_results_regs_arQ, type="latex", file="har_analysis_arQ.tex")
print.xtable(xTable_results_regs_har, type="latex", file="har_analysis_har.tex")
print.xtable(xTable_results_regs_haQ, type="latex", file="har_analysis_haQ.tex")
print.xtable(xTable_results_regs_hQL, type="latex", file="har_analysis_hQL.tex")

xTable_rmse_eMat_collect  <- xtable(rmse_eMat_collect, caption = "HAR forecasts",align=c("l","c","c","c","c","c"),digits = 4)
print.xtable(xTable_rmse_eMat_collect, type="latex", file=paste(c("har_forecasts_s",fsteps,".tex"),collapse = ""))



