function_data_filter<-function(fdata){
  
  fdata   <- apply(fdata[,-1],2,as.numeric)
  fdata   <- interpNA(fdata, method="before") 
  fncheck <- NCOL(fdata)
  
  for (fc in 4:fncheck){
    if (is.na(fdata[NROW(fdata),fc])){
      fdata_fc_WONA           <- removeNA(fdata[,fc])
      fdata[NROW(fdata),fc]   <- fdata_fc_WONA[NROW(fdata_FC_WONA)]
    }
  }
  
  fdata  <- interpNA(fdata, method="before")    # To handle if there are multiple NAs in the end.
  fdata  <- removeNA(fdata)                     # To handle if there is NA during the first observation.
  
  return(fdata)
  
}

function_dateX_construction <- function(fdateM){
  
  fdateXmat   <- matrix(0,nrow(fdateM),5)
  
  for (ft in 1:nrow(fdateM)){
    fdatett         <- paste(c(fdateM[ft,2],"/",fdateM[ft,3],"/",fdateM[ft,1]),collapse = "")
    fdateConv2      <- as.POSIXlt(fdatett, format="%m/%d/%Y")
    fdateXmat[ft,]  <- t(c((fdateConv2$year+1900),(fdateConv2$mon+1),fdateConv2$mday,(fdateConv2$yday+1),fdateConv2$wday ) )
  }  
  
  fdateX  <- fdateXmat[,1] + fdateXmat[,4]/366
  
  return(fdateX)
  
}

function_sumstats<-function(fret){
  
  fmean   <- mean(fret)
  fstd    <- stdev(fret)
  fskew   <- rowSkewness(t(fret))
  fkur    <- rowKurtosis(t(fret)) + 3     # If 3 is not added, then it will be in excess to the Gaussian case,
  fSR     <- fmean/fstd
  
  return(c(fmean,fstd,fskew,fkur,fSR))
  
}

function_sumstatsbyticker<-function(fretM,fscale){
  
  fN            <- ncol(fretM)
  fscalevec     <- c(fscale,sqrt(fscale),1,1,sqrt(fscale))
  
  for (fnn in 1:fN){
    fretSR      <- fretM[,fnn] 
    fsumR       <- function_sumstats(fretSR)
    fstoreR     <- t(fsumR)*t(fscalevec)
    if (fnn == 1){
      fstore    <- fstoreR 
    } else{
      fstore    <- rbind(fstore,fstoreR)  
    }
  }
  
  return(fstore)
  
}

function_sumstatsbyticker_aug<-function(fretM,fscale){
  
  fN            <- ncol(fretM)
  fscalevec     <- c(fscale,sqrt(fscale),1,1,sqrt(fscale),sqrt(fscale),sqrt(fscale),1)
  
  for (fnn in 1:fN){
    fretSR      <- fretM[,fnn] 
    fsumR       <- function_sumstats(fretSR)
    fvHAC       <- function_HAC_estimation_Parzen(fretSR,"stdev")
    fretACF     <- acf(fretSR, lag.max = 5, plot = FALSE)
    fretACFS    <- (fretACF$acf)[2]
    fsumR       <- c(fsumR,fvHAC,fsumR[1]/fvHAC,fretACFS)
    fstoreR     <- t(fsumR)*t(fscalevec)
    
    if (fnn == 1){
      fstore    = fstoreR 
    } else{
      fstore    = rbind(fstore,fstoreR)  
    }
  }
  
  return(fstore)
  
}


function_HAC_estimation_Parzen <- function(fret,fstat){
  
  fobs    <- NROW(fret)
  flags   <- floor(4*(fobs/100)^(2/9))
  fxuux   <- var(fret)
  
  for (fl in 1:flags){
    fza   <- sum(fret[(fl+1):fobs] * fret[1:(fobs-fl)])/fobs
    
    if ((fl/flags)<=1/2){
      fweight  <- (1-6*(fl/flags)^2+6*(fl/flags)^3)
    }
    
    if ((fl/flags)>1/2){
      fweight  <- 2*(1-(fl/flags))^3
    }
    
    fxuux   <- fxuux + 2*fweight*fza
  }
  
  fLRV  <- fxuux
  fSE   <- sqrt(fLRV)
  
  if(fstat == "variance"){
    fretStat <- fLRV  
  }
  
  if(fstat == "stdev")
  {
    fretStat <- fSE  
  }
  
  return(fretStat)
  
}

function_distress_plot1  <- function(fdates,fdata,favg,fstat,fbounds)
{
  fpseudo_df  <- data.frame(year  = fdates, 
                            value = fdata[,1], 
                            lower = fdata[,"lower"], 
                            upper = fdata[,"upper"])
  
  fplot <- ggplot(fpseudo_df,aes(year, value)) +
    geom_point(shape=21, size=3, fill="blue")+
    geom_hline(yintercept = 0,colour="brown") +
    geom_hline(yintercept = favg,colour="orange",size=1,linetype = "dashed") +
    geom_errorbar(aes(ymin=lower, ymax=upper), width=0.3) +
    ylim(fbounds[1],fbounds[2]) + 
    theme(axis.text.x = element_text(angle=65, vjust=0.6)) +
    labs(title=paste(c(fstat," during market distress"),collapse = ""), y = fstat, x = "dates")
  
  return(fplot)
  
}

function_distress_plot2  <- function(fdates,fdata,favg,fstat,fbounds,fticker)
{
  fpseudo_df  <- data.frame(year  = fdates, 
                            value = fdata[,1], 
                            lower = fdata[,"lower"], 
                            upper = fdata[,"upper"])
  
  fplot <- ggplot(fpseudo_df,aes(year, value)) +
    geom_point(shape=21, size=3, fill="blue")+
    geom_hline(yintercept = 0,colour="brown") +
    geom_hline(yintercept = favg,colour="orange",size=1,linetype = "dashed") +
    geom_errorbar(aes(ymin=lower, ymax=upper), width=0.3) +
    ylim(fbounds[1],fbounds[2]) + 
    theme(axis.text.x = element_text(angle=65, vjust=0.6)) +
    labs(title=paste(c(fstat," during market distress: ",fticker),collapse = ""), y = fstat, x = "dates")
  
  return(fplot)
  
}

function_predictive_regression_evaluation<-function(fYvar,fXvar)
{
  
  fobs    <- NROW(fYvar)
  fxmat   <- cbind(matrix(1,fobs,1),fXvar)
  fcol    <- NCOL(fxmat)-1
  fbmat   <- inv(t(fxmat) %*% fxmat) %*% (t(fxmat) %*% fYvar)
  fres    <- fYvar - fxmat %*% fbmat
  
  # Computing explanatory power
  fSStot  <- sum(demean(fYvar)^2)
  fSSres  <- sum(fres^2)
  fR2     <- 1-fSSres/fSStot
  fR2     <- 1-(1-fR2)*(fobs-1)/(fobs-fcol-1-1)
  
  # Computing Robust Inference
  flags   <- floor(4*(fobs/100)^(2/9))
  ficovX  <- inv(t(fxmat) %*% fxmat)
  fhhat   <- fxmat * (fres %*% matrix(1,1,(fcol+1)))
  fxuux   <- t(fhhat) %*% fhhat
  
  for (fl in 1:flags)
  {
    fza   <- t(fhhat[(fl+1):fobs,]) %*% fhhat[1:(fobs-fl),]
    if ((fl/flags)<=1/2) # Parzen weights
    {
      fweight   <- (1-6*(fl/flags)^2+6*(fl/flags)^3)
    }else if ((fl/flags)>1/2)
    {
      fweight   <- 2*(1-(fl/flags))^3
    }
    fxuux     <- fxuux + fweight*(fza+t(fza))
  }
  
  fcov      <- ficovX %*% fxuux %*% ficovX
  fSE       <- diag(fcov)^(1/2)
  
  fresACF   <- acf(fres, lag.max = 50, plot = FALSE)
  fresACFS  <- fresACF$acf
  fresACFF  <- c(fresACFS[2],mean(fresACFS[3:12]),mean(fresACFS[13:22]))
  
  return(c(fbmat,(fbmat/fSE),fR2,fresACFF))
  
}

function_hedge_efficacy <-function(fbetaMat1,fbetaMat2,flabel,fstat,fcurrencies)
{
  
  fBetaAvgTick    <- rep(fcurrencies,2)
  fBetaAvgMatt    <- c(fbetaMat1,fbetaMat2)
  fBetaAvgType    <- c(rep(flabel[1],NROW(fcurrencies)),rep(flabel[2],NROW(fcurrencies)))
  fBetaAvgData    <- data.frame(fBetaAvgTick,fBetaAvgMatt)
  
  fBetaPlot   = ggplot(fBetaAvgData, aes(fBetaAvgTick,fBetaAvgMatt)) +
    geom_bar(stat="identity", aes(fill = fBetaAvgType), position = "dodge",colour="black",width = 0.8) + 
    labs(title=paste(c("Significant deviation from mean: ",fstat),collapse = ""), 
         y = "percentage of days", x = element_blank()) + 
    scale_fill_manual("Direction",values=c('red','green')) +
    theme(axis.text.x = element_text(angle=65, vjust=0.6))
  
  print(fBetaPlot)
  
  return(fBetaPlot)
  
}

function_4series_conf_plot1 <-function(fxvar,fyMat,flabel,faxis,ftitle)
{
  fybounds  <- c(min(fyMat),max(fyMat))  
  
  fplot1= ggplot(mapping = aes(x=fxvar)) +
    geom_line(mapping = aes(y=fyMat[,1],colour=flabel[1]), size=0.85) + 
    geom_line(mapping = aes(y=fyMat[,2],colour=flabel[2]), size=0.85) + 
    geom_line(mapping = aes(y=fyMat[,3],colour=flabel[3]), size=0.85) + 
    geom_line(mapping = aes(y=fyMat[,4],colour=flabel[4]), size=0.85) + 
    ylim(0.7, 1) +
    geom_hline(yintercept = 0,colour="brown") +
    geom_hline(yintercept = 0.95,colour="brown") +
    scale_colour_manual("spot measure: ", 
                        breaks = c(flabel[1], flabel[2], flabel[3], flabel[4]),
                        values = c("black", "blue","red","green")) +
    theme(legend.position = "bottom",legend.key.size = unit(1, "cm")) +
    labs(title=ftitle,
         y = faxis[2],
         x = faxis[1])
  
  return(fplot1)
}

function_4series_conf_plot2 <-function(fxvar,fyMat,flabel,faxis,ftitle)
{
  fybounds  <- c(min(fyMat),max(fyMat))  
  
  fplot1= ggplot(mapping = aes(x=fxvar)) +
    geom_line(mapping = aes(y=fyMat[,1],colour=flabel[1]), size=0.85) + 
    geom_line(mapping = aes(y=fyMat[,2],colour=flabel[2]), size=0.85) + 
    geom_line(mapping = aes(y=fyMat[,3],colour=flabel[3]), size=0.85) + 
    geom_line(mapping = aes(y=fyMat[,4],colour=flabel[4]), size=0.85) + 
    ylim(0.5, 1) +
    geom_hline(yintercept = 0,colour="brown") +
    geom_hline(yintercept = 0.95,colour="brown") +
    scale_colour_manual("spot measure: ", 
                        breaks = c(flabel[1], flabel[2], flabel[3], flabel[4]),
                        values = c("black", "blue","red","green")) +
    theme(legend.position = "bottom",legend.key.size = unit(1, "cm")) +
    labs(title=ftitle,
         y = faxis[2],
         x = faxis[1])
  
  return(fplot1)
}


